import os
try:
    import unittest2 as unittest
except:
    import unittest
from rock.config import Config


TESTS_PATH = os.path.join(os.path.dirname(__file__))
ENV_PATH = os.path.join(TESTS_PATH, 'assets', 'env')


def setenv(mount='test', data='test'):
    @staticmethod
    def mount_path(*args):
        return os.path.join(*(ENV_PATH, mount) + args)

    @staticmethod
    def data_path(*args):
        return os.path.join(*(TESTS_PATH, 'assets', 'data', data) + args)

    Config.mount_path = mount_path
    Config.data_path = data_path



import argparse
import os
import sys
from rock.exceptions import Error
from rock.project import Project
from rock.runtime import list as runtime_list

stdout = sys.stdout


def project(args):
    config = {'path': args.path}
    if args.verbose:
        config['verbose'] = True
    if args.dry_run:
        config['dry_run'] = True
        config['verbose'] = True
    if args.runtime:
        config['runtime'] = args.runtime
    return Project(config, env=args.env)


def build(args, extra):
    project(args).build(*extra)


def clean(args, extra):
    project(args).clean(*extra)


def create(args, extra):
    names = project(args).create(args.name, *extra)
    if names:
        stdout.write('%s\n' % '\n'.join(names))


def env(args, extra):
    for name, value in project(args).config['env'].items():
        stdout.write('export %s="%s"\n' % (name, value))


def runtime(args, extra):
    for r in runtime_list():
        stdout.write('%s\n' % r.name)


def run(args, extra):
    project(args).run(extra)


def test(args, extra):
    project(args).test(*extra)


def main(args=None):
    description = """
    rock helps you build, test and run your app in the Rock Platform.
    """

    parser = argparse.ArgumentParser(prog='rock', description=description)

    # general options
    parser.add_argument('--dry-run', action='store_true',
                        help="show commands, but don't run")
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='show run commands')

    # project options
    project_options = parser.add_argument_group('project options')
    project_options.add_argument('--path', help='set path',
                                 default=os.getcwd())
    project_options.add_argument('--env', help='set env',
                                 default=os.environ.get('ROCK_ENV', 'local'))
    project_options.add_argument('--runtime', help='set runtime')

    # project commands
    project = parser.add_subparsers(title='project')

    # project: build
    parser_build = project.add_parser('build', help='build project')
    parser_build.set_defaults(func=build)

    # project: clean
    parser_clean = project.add_parser('clean', help='clean project')
    parser_clean.set_defaults(func=clean)

    # project: create
    parser_create = project.add_parser('create', help='create project')
    parser_create.set_defaults(func=create)
    parser_create.add_argument('name', nargs="?", help='template name')

    # project: env
    parser_env = project.add_parser('env', help='show environment variables')
    parser_env.set_defaults(func=env)

    # runtime
    parser_runtime = project.add_parser('runtime', help='list runtimes',
                                        add_help=False)
    parser_runtime.set_defaults(func=runtime)

    # project: run
    parser_run = project.add_parser('run', help='run project file',
                                    add_help=False)
    parser_run.set_defaults(func=run)

    # project: test
    parser_test = project.add_parser('test', help='test project')
    parser_test.set_defaults(func=test)

    try:
        args, extra = parser.parse_known_args(args)
        args.func(args, extra)
    except Error, error:
        message = '%s' % error
        if not message.endswith('\n'):
            message += '\n'
        parser.exit(1, message)



import helper
from rock import utils


class UtilsTestCase(helper.unittest.TestCase):

    def test_shell(self):
        def execute(self, *args):
            self.args = args
        utils.Shell.execute = execute
        s = utils.Shell()
        self.assertTrue(isinstance(s.__enter__(), utils.Shell))
        s.write('ok')
        self.assertEqual(s.stdin.getvalue(), 'ok\n')
        def execl(*args):
            self.assertEqual(len(args), 5)
            self.assertEqual(args[0], '/usr/bin/env')
            self.assertEqual(args[1], 'bash')
            self.assertEqual(args[2], 'bash')
            self.assertEqual(args[3], '-c')
            self.assertEqual(args[4], 'ok\n')
        utils.os.execl = execl
        s.__exit__('type', 'value', 'tracebook')



import os
from rock.config import Config


class Runtime(object):

    def __init__(self, name):
        self.name = name

    @staticmethod
    def root_path(*args):
        return Config.mount_path(*('opt', 'rock', 'runtime') + args)

    def path(self, *args):
        return self.root_path(*(self.name,) + args)

    def exists(self, *args):
        return os.path.exists(self.path(*args))


def list():
    path = Runtime.root_path()
    if not os.path.isdir(path):
        return []
    rs = map(Runtime, os.listdir(path))
    rs = [r for r in rs if r.exists('rock.yml')]
    return sorted(rs, key=lambda r: r.name)



import os
import pipes
import re
import string
import sys
import yaml
from rock.config import Config
from rock.exceptions import ConfigError
from rock.process import ProcessManager
from rock.utils import Shell

NAME_RE = re.compile('^[a-zA-Z_]+$')


class Project(object):

    def __init__(self, *args, **kwargs):
        self.config = Config(*args, **kwargs)

    def _setup(self, shell):
        # declare builtin functions
        shell.write('warn() { echo "$@" >&2; }')
        shell.write('die() { warn "$@"; exit 1; }')
        # print commands as they're run
        if self.config.get('verbose'):
            shell.write('set -o verbose')
        # don't execute commands, just print them
        if self.config.get('dry_run'):
            shell.write('set -o noexec')
        # exit with error if any one command fails
        shell.write('set -o errexit')
        # switch to project directory
        shell.write('cd ' + pipes.quote(self.config['path']))
        # setup environment variables
        if self.config.get('env'):
            # blank line before exports
            shell.write('')
            for name, value in self.config['env'].items():
                shell.write('export %s="%s"' % (name, value))
        # blank line before command
        shell.write('')

    def execute(self, command):
        with Shell() as shell:
            self._setup(shell)
            # run command and wait for results
            if isinstance(command, (list, tuple)):
                shell.write(' '.join(command))
            else:
                shell.write(command)

    def execute_type(self, name, *args):
        section = '%s_%s' % (name, args[0]) if len(args) > 0 else name
        if section not in self.config:
            raise ConfigError('section not found: %s' % section)
        self.execute(self.config[section])

    def execute_section(self, name):
        section = self.config.get(name)

        if isinstance(section, dict):
            pm = ProcessManager()
            for name, value in section.items():
                pm.add_process(name, '%s run %s' %
                               (sys.argv[0], pipes.quote(value)))
            pm.loop()
        else:
            self.execute(section)

    def build(self, *args):
        self.execute_type('build', *args)

    def clean(self, *args):
        self.execute_type('clean', *args)

    def _template(self, path, args):
        with Shell() as shell:
            self._setup(shell)
            # handle options
            length = len(args)
            skip_next = False
            template_args = []
            for i in xrange(length):
                if skip_next:
                    skip_next = False
                    continue
                arg = args[i]
                if arg.startswith('--'):
                    name, value = arg[2:], '1'
                    if '=' in name:
                        name, value = name.split('=', 1)
                    elif (i + 1 < length and not args[i + 1].startswith('--')):
                        value = args[i + 1]
                        skip_next = True
                    name = name.upper().replace('-', '_')
                    if NAME_RE.match(name):
                        # write options to shell
                        shell.write('export ROCK_%s="%s"' %
                                    (name, pipes.quote(value)))
                    else:
                        raise ConfigError('invalid argument: ' + arg)
                else:
                    template_args.append(arg)
            shell.write('. ' + path)

    def create(self, name=None, *args):
        if not self.config._setup:
            self.config._setup = True
        # project path
        path = self.config['path']
        # if name is given try to run it, otherwise list available templates
        if name:
            # create project path if it doesn't exist
            if not os.path.exists(path):
                os.makedirs(path)
            # ensure its a directory
            if not os.path.isdir(path):
                raise ConfigError('path must be a directory')
            # ensure its empty
            if os.listdir(path):
                raise ConfigError('directory is not empty')
            # search for template and run if it exists
            for template_path in self.config.paths('template'):
                template_path = os.path.join(template_path, name)
                if os.path.isfile(template_path):
                    # run template
                    self._template(template_path, args)
        else:
            templates = set()
            # get paths
            for path in self.config.paths('template'):
                if not os.path.isdir(path):
                    continue
                for template in os.listdir(path):
                    if os.path.isfile(os.path.join(path, template)):
                        templates.add(template)
            templates = list(templates)
            templates.sort()
            return templates

    def run(self, args):
        if len(args) == 0 and 'run' in self.config:
            self.execute_section('run')
        elif len(args) == 1 and 'run_%s' % args[0] in self.config:
            self.execute_section('run_%s' % args[0])
        else:
            self.execute(args)

    def test(self, *args):
        self.execute_type('test', *args)



import helper
import os
import subprocess
import tempfile
from StringIO import StringIO
from rock import cli, utils
from rock.exceptions import ConfigError
from rock.project import Project


class Args(object):

    def __init__(self):
        self.path = os.path.join(helper.TESTS_PATH, 'assets', 'project', 'simple')
        self.verbose = True
        self.dry_run = True
        self.runtime = 'test123'
        self.name = ''
        self.env = 'local'


class CliTestCase(helper.unittest.TestCase):

    def setUp(self):
        helper.setenv()
        def execl(*args):
            self.args = args
        utils.os.execl = execl
        self.stdout = cli.stdout = StringIO()

    def test_project(self):
        self.assertTrue(isinstance(cli.project(Args()), Project))

    def test_build(self):
        cli.build(Args(), ['deployment'])
        self.assertTrue('\n\nbuild deployment\n\n' in self.args[4])

    def test_clean(self):
        cli.clean(Args(), [])
        self.assertTrue('\n\nclean\n\n' in self.args[4])

    def test_create(self):
        args = Args()
        args.name = 'test-something'
        path = tempfile.mkdtemp('rock.tests')
        try:
            # ok
            args.path = os.path.join(path, 'one')
            cli.create(args, ['--one', 'one', '--two=two', 'arg'])
            self.assertTrue('/test-something' in self.args[4])
            # bad args
            self.assertRaises(ConfigError, cli.create, args, ['--f:ail=true'])
            # path not empty
            with open(os.path.join(args.path, 'test'), 'w+') as f:
                f.write('test')
            self.assertRaises(ConfigError, cli.create, args, [])
            # path not dir
            args.path = os.path.join(path, 'two')
            with open(args.path, 'w+') as f:
                f.write('test')
            self.assertRaises(ConfigError, cli.create, args, [])
        finally:
            subprocess.check_call(['rm', '-fr', path])

    def test_create_list(self):
        cli.create(Args(), [])
        self.assertEqual(self.stdout.getvalue(), 'test-something\n')

    def test_env(self):
        cli.env(Args(), [])
        self.assertTrue('\nexport TEST_PATH="test_path"\n' in self.stdout.getvalue())

    def test_runtime(self):
        cli.runtime(Args(), [])
        self.assertTrue('\ntest123\n' in self.stdout.getvalue())

    def test_run(self):
        cli.run(Args(), ['one', 'two', 'three'])
        self.assertTrue('\none two three\n' in self.args[4])
        # str section
        cli.run(Args(), [])
        self.assertTrue('\necho zero\n' in self.args[4])
        # dict section
        cli.run(Args(), ['full'])

    def test_test(self):
        cli.test(Args(), [])
        self.assertTrue('\n\ntest\n\n' in self.args[4])
        # not found
        self.assertRaises(ConfigError, cli.test, Args(), ['not_found'])

    def test_main(self):
        cli.main(args=['runtime'])
        self.assertRaises(SystemExit, cli.main, ['--path=%s/not-found' % helper.TESTS_PATH, 'run', 'ok'])



from distutils.core import setup
from rock import __version__ as version

long_description = """
This is a command line tool that simplifies building, testing, and running
applications on the Rock Platform.

See http://www.rockplatform.org for more information.
"""

setup(
    name='rock',
    version=version,
    description='Rock Platform tool',
    long_description=long_description.strip(),
    author='Silas Sewell',
    author_email='silas@sewell.org',
    license='MIT',
    url='https://github.com/rockplatform/rock',
    packages=['rock'],
    package_data={'rock': [
        'data/*/*',
    ]},
    scripts=['scripts/rock'],
    install_requires=[
        'PyYAML',
        'argparse',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
)
